
//+------------------------------------------------------------------+
//|                                      Williams? Percent Range.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2005, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net/"




#property indicator_separate_window
#property indicator_minimum -100
#property indicator_maximum 0
#property indicator_buffers 1
#property indicator_color1  DodgerBlue
#property indicator_level1 -20
#property indicator_level2 -80




extern string TimeFrame       = "Current time frame";
extern int    WprPeriod       = 14;
extern int    lowlevel        = -80;
extern int    highlevel       = -20;
extern bool   Interpolate     = true;

extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;





double wpr[];
double trend[];


string IndicatorFileName;
bool   Calculatewpr;
bool   ReturningBars;
int    timeFrame;





int init()
  {
   IndicatorBuffers(2);
   SetIndexBuffer(0,wpr);
   SetIndexBuffer(1,trend);
   
   
   IndicatorFileName = WindowExpertName();
   Calculatewpr      = (TimeFrame=="Calculatewpr"); if (Calculatewpr) return(0);
   ReturningBars     = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
   timeFrame         = stringToTimeFrame(TimeFrame);
   
   
   
   IndicatorShortName(timeFrameToString(timeFrame)+ " %R (" +WprPeriod+ ")");


   return(0);
  }
//+------------------------------------------------------------------+
//| Williams? Percent Range                                          |
//+------------------------------------------------------------------+
int start()
  {
   int counted_bars=IndicatorCounted();
   int i,r,limit;
   
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1);
           if (ReturningBars)  { wpr[0] = limit+1; return(0); }

   
   
      if (Calculatewpr || timeFrame==Period())
      {
          
      for(i=limit; i>=0; i--)
      {
   
      double dMaxHigh = High[Highest(NULL, 0, MODE_HIGH,WprPeriod, i)];
      double dMinLow  = Low[Lowest(NULL,   0, MODE_LOW, WprPeriod, i)];      
              wpr[i]  = -100*(dMaxHigh - Close[i]) / (dMaxHigh - dMinLow);
              trend[i]= trend[i+1];
      
      if (wpr[i] < lowlevel  && wpr[i+1] >= lowlevel)  trend[i] =  1;
      if (wpr[i] > highlevel && wpr[i+1] <= highlevel) trend[i] = -1;
        
      }
      manageAlerts(); 
      return(0);
   }
           
    
      limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
      for(i=limit; i>=0; i--)
      {
      int y    = iBarShift(NULL,timeFrame,Time[i]);
      wpr[i]   = iCustom(NULL,timeFrame,IndicatorFileName,"Calculatewpr",WprPeriod,lowlevel,highlevel,0,y);
      trend[i] = iCustom(NULL,timeFrame,IndicatorFileName,"Calculatewpr",WprPeriod,lowlevel,highlevel,1,y);
      
      
      if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         

         datetime time = iTime(NULL,timeFrame,y);
         for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
         double factor = 1.0 / n;
         for(int k = 1; k < n; k++)
         {
  	      wpr[i+k] = k*factor*wpr[i+n] + (1.0-k*factor)*wpr[i];
  	      }  	            
   }
      
   manageAlerts();
      
   return(0);
}


void manageAlerts()
{
   if (!Calculatewpr && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"buy");
         if (trend[whichBar] ==-1) doAlert(whichBar,"sell");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," - ",timeFrameToString(timeFrame)+" wpr trend changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," wpr "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}


